@@ -54,12 +54,15 @@ class QuantizedReLU(keras.layers.Layer):
         self.max_value = max_value
 
     def call(self, inputs):
-        """ReLU activation function. In other terms:
+        """ReLU activation function.
+
+        In other terms:
+
             1. clip the value between 0 and :attr:`max_value`.
             2. quantize the output if an output_quantizer is set.
 
         Args:
-            inputs (:obj:`FixedPoint` or :obj:`tf.Tensor`): the inputs tensor.
+            inputs (:obj:`QTensor`): the inputs tensor.
 
         Returns:
             :obj:`FixedPoint`: QuantizedReLU outputs.
@@ -88,5 +91,5 @@ class QuantizedReLU(keras.layers.Layer):
     def get_config(self):
         config = super().get_config()
         config.update({"max_value": self.max_value})
-        config["quant_config"] = self.quant_config
+        config.update({"quant_config": self.quant_config})
         return config
