@@ -20,8 +20,8 @@ import tensorflow as tf
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH, round_log2, pow2
 from .reshaping import QuantizedReshape, QuantizedPermute
 from .shiftmax import shiftmax, QuantizedShiftmax
-from .calibrable import Calibrable, CalibrableVariable
-from .quantizers import deserialize_quantizer, OutputQuantizer
+from .recorders import TensorRecorder
+from .quantizers import OutputQuantizer
 
 
 __all__ = ["Attention", "string_to_softmax", "QuantizedAttention"]
@@ -116,7 +116,7 @@ class Attention(keras.layers.Layer):
         x = keras.layers.Reshape((-1, self.num_heads, self.dim))(x)
         return keras.layers.Permute((2, 1, 3))(x)
 
-    def call(self, inputs, training=None):
+    def call(self, inputs):
         # Separate 2D embeddings per head to obtain 3D inputs
         query = self.separate_heads(inputs[0])
         key = self.separate_heads(inputs[1])
@@ -150,25 +150,28 @@ class Attention(keras.layers.Layer):
 
 
 @tf.keras.utils.register_keras_serializable()
-class QuantizedAttention(Calibrable, Attention):
+class QuantizedAttention(Attention):
     """An Attention layer that operates on quantized inputs
 
     Args:
         num_heads (int): the number of attention heads
+        quant_config (dict): the serialized quantization configuration.
         softmax (str, optional): 'softmax' or 'shiftmax'. Defaults to 'shiftmax'
-        quant_config (dict, optional): the serialized quantization configuration.
-            Defaults to empty configuration.
     """
 
-    def __init__(self, num_heads, softmax='shiftmax', quant_config={}, **kwargs):
+    def __init__(self, num_heads, quant_config, softmax='shiftmax', **kwargs):
         if softmax != 'shiftmax':
             raise ValueError(
                 "Only shiftmax is supported for quantized attention")
         super().__init__(num_heads=num_heads, softmax=softmax, **kwargs)
         self.quant_config = quant_config
 
-        self.out_quantizer = deserialize_quantizer(
-            self.quant_config, "output_quantizer", OutputQuantizer, False)
+        out_quant_cfg = quant_config.get("output_quantizer", False)
+        if out_quant_cfg:
+            self.out_quantizer = OutputQuantizer(
+                name="output_quantizer", **out_quant_cfg)
+        else:
+            self.out_quantizer = None
 
         # Override softmax operation
         softmax_quant_conf = self.quant_config.get("softmax", {})
@@ -178,21 +181,21 @@ class QuantizedAttention(Calibrable, Attention):
         self.buffer_bitwidth = self.quant_config.get(
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         # Add objects that will store the shift values.
-        self.values_shift = CalibrableVariable()
+        self.values_shift = TensorRecorder()
 
     def separate_heads(self, x):
         x = QuantizedReshape((-1, self.num_heads, self.dim))(x)
         return QuantizedPermute((2, 1, 3))(x)
 
-    def call(self, inputs, training=None):
+    def call(self, inputs):
         if any(not isinstance(x, FixedPoint) for x in inputs):
             # If any of the inputs is not a FixedPoint, raise an error
             raise ValueError("QuantizedAttention only accepts FixedPoint inputs")
         # Separate 2D embeddings per head to obtain 3D inputs
-        # Note: values is allowed to be quantized per-axis, so it must be aligned
         query = self.separate_heads(inputs[0])
         key = self.separate_heads(inputs[1])
-        value, vshift = inputs[2].promote(self.buffer_bitwidth).align()
+        # Expand the values to a higher bitwidth to avoid saturation and align them
+        value, vshift = inputs[2].expand(self.buffer_bitwidth)
         self.values_shift(vshift)
         value = self.separate_heads(value)
         # Promote query to avoid saturation
@@ -212,7 +215,7 @@ class QuantizedAttention(Calibrable, Attention):
         output = QuantizedReshape((-1, self.hidden_size))(output)
         # Refine output bitwidth precision if needed
         if self.out_quantizer is not None:
-            output = self.out_quantizer(output, training=training)
+            output = self.out_quantizer(output)
         return output, mask
 
     def get_config(self):
