@@ -17,7 +17,7 @@
 import keras
 import tensorflow as tf
 
-from ..tensors import FixedPoint
+from ..tensors import QTensor
 
 
 __all__ = ["QDropout"]
@@ -30,10 +30,10 @@ class QDropout(keras.layers.Dropout):
     It is only implemented as a passthrough.
     """
 
-    def call(self, inputs, training=None):
-        # raise an error if the inputs are not FixedPoint
-        if not isinstance(inputs, FixedPoint):
-            raise TypeError(f"QDropout only accepts FixedPoint inputs.\
+    def call(self, inputs):
+        # raise an error if the inputs are not QTensor
+        if not isinstance(inputs, QTensor):
+            raise TypeError(f"QDropout only accepts QTensor inputs.\
                              Receives {type(inputs)} inputs.")
 
         # QuantizedDropout act as a pass through.
