@@ -54,12 +54,8 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
     def build(self, input_shape):
         super().build(input_shape)
 
-        # Save an approximation of nb_channels as PoT to calculate the mad of inputs as:
-        # mad = sum(abs(inputs)) * 2 ** -mad_shift
         if len(self.axis) > 1:
             raise ValueError(f"{self.__class__.__name__} does not support multiple axis.")
-        nb_channels = tf.cast(input_shape[self.axis[0]], tf.float32)
-        self.mad_shift = tf.math.round(tf.math.log(nb_channels) / tf.math.log(2.))
 
     def _get_mad(self, inputs, axis, *args, name=None, **kwargs):
         """Compute the mean absolute deviation (mad).
@@ -74,13 +70,11 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
         Returns:
             :obj:`tensorflow.Tensor`: mean absolute deviation (mad).
         """
-        # Compute the mad, taking into account the approximations:
-        # mad \approx sum(abs(inputs))
-        # nb_channels \approx 2 ** round(log2(inputs.shape[axis]))
-        nb_channels_inv = 2.0 ** -self.mad_shift
+        # Compute the mad
         with ops.name_scope(name, "_mad", [inputs, axis]):
             mad = tf.math.reduce_sum(tf.math.abs(inputs), axis, *args, **kwargs, name="mad")
-            mad = mad * nb_channels_inv
+            nb_channels = tf.cast(inputs.shape[self.axis[0]], tf.float32)
+            mad = mad / nb_channels
         return mad
 
     def call(self, inputs):
@@ -138,9 +132,7 @@ class QuantizedLayerNormalization(LayerMadNormalization):
             self.out_quantizer = None
         self.buffer_bitwidth = self.quant_config.get(
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
-        self.mad_rec_frac_bits = self.quant_config.get("mad_rec_frac_bits", 16)
         assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
-        assert self.mad_rec_frac_bits >= 0, "The mad_rec_frac_bits must be a non-negative integer."
         self.gamma_quantizer = WeightQuantizer(
             name="gamma_quantizer", **quant_config["gamma_quantizer"])
         self.beta_quantizer = WeightQuantizer(
@@ -149,7 +141,6 @@ class QuantizedLayerNormalization(LayerMadNormalization):
         # Add objects that will store the shift values.
         self.input_shift = TensorRecorder()
         self.x_rescaled_shift = TensorRecorder()
-        self.one_bits = TensorRecorder()
         self.beta_shift = TensorRecorder()
         self.output_shift = TensorRecorder()
 
@@ -176,14 +167,15 @@ class QuantizedLayerNormalization(LayerMadNormalization):
         # Compute the Absolute Deviation (MAD)
         mad_sum = tf.math.reduce_sum(
             inputs.abs(), self.axis, keepdims=True, name="mad")
-        # Calculate a gamma_rescaled = gamma * 2 ** mad_shift. This is the
+        # Calculate a gamma_rescaled = gamma * nb_channels. This is the
         # value that will be quantized.
-        gamma_rescaled = self.gamma * (2 ** self.mad_shift)
+        nb_channels = tf.cast(inputs.shape[self.axis[0]], tf.float32)
+        gamma_rescaled = self.gamma * nb_channels
         # Quantize the learnable parameters gamma_rescaled and beta
         gamma_rescaled = self.gamma_quantizer(gamma_rescaled)
         beta = self.beta_quantizer(self.beta)
 
-        # Calculate the gamma_on_mad_sum = (gamma * 2**mad_shift) / mad_sum.
+        # Calculate the gamma_on_mad_sum = (gamma * nb_channels) / mad_sum.
         # This can be evaluated once on hardware.
         gamma_on_mad_sum = gamma_rescaled / mad_sum
 
