@@ -16,8 +16,8 @@
 # ******************************************************************************
 import tensorflow as tf
 
-from ...tensors import QTensor, QFloat, FixedPoint
-from ..recorders import TensorRecorder, FixedPointRecorder, QFloatRecorder
+from ...tensors import QFloat
+from ..recorders import TensorRecorder, QFloatRecorder
 from .quantizers import Quantizer
 
 
@@ -42,9 +42,7 @@ class AlignedWeightQuantizer(Quantizer):
         super().__init__(bitwidth, signed, **kwargs)
         # Add the object that will store the shift values.
         self.shift = TensorRecorder()
-        # We don't know until the first call the type of the quantized weights
-        # so we need to wait before initializing their recorder.
-        self.qweights = None
+        self.qweights = QFloatRecorder()
 
     def call(self, inputs, other):
         """Quantize the float inputs, aligned on another QFloat
@@ -58,47 +56,32 @@ class AlignedWeightQuantizer(Quantizer):
 
         Args:
             inputs (:obj:`tensorflow.Tensor`): the inputs tensor.
-            other (:obj:`QTensor`): a tensor to align on.
+            other (:obj:`QFloat`): a tensor to align on.
 
         Returns:
-            :obj:`QTensor`: a quantized tensor of the same types than other.
+            :obj:`QFloat`: a quantized tensor with the same scales and frac_bits as other.
         """
         if not isinstance(inputs, (tf.Tensor, tf.Variable)):
             raise ValueError(
                 f"{type(inputs)} as first param is not supported."
                 "AlignedWeightQuantizer only accepts tf.Tensor.")
-        if not isinstance(other, QTensor):
+        if not isinstance(other, QFloat):
             raise ValueError(
                 f"{type(other)} as second param is not supported."
-                "AlignedWeightQuantizer only accepts QTensor types.")
+                "AlignedWeightQuantizer only accepts QFloat.")
 
-        if isinstance(other, FixedPoint):
-            other_value_bits = other.value_bits
-            other_frac_bits = other.frac_bits
-            if self.qweights is None:
-                self.qweights = FixedPointRecorder()
-        else:
-            other_value_bits = other.fp.value_bits
-            other_frac_bits = other.fp.frac_bits
-            if self.qweights is None:
-                self.qweights = QFloatRecorder()
+        other_value_bits = other.fp.value_bits
+        other_frac_bits = other.fp.frac_bits
         # Compute the symmetric quantization ranges from the inputs
         ranges = tf.math.reduce_max(tf.math.abs(inputs))
         # Evaluate the maximum fractional bits we can use for the specified ranges
-        if isinstance(other, FixedPoint):
-            frac_bits = FixedPoint.max_frac_bits(self.value_bits, ranges)
-        else:
-            frac_bits = QFloat.max_frac_bits(self.value_bits, ranges, other.scales)
+        frac_bits = QFloat.max_frac_bits(self.value_bits, ranges, other.scales)
         # Remove the input-dependent fractional bits from the gradient tape to avoid a loopback
         frac_bits = tf.stop_gradient(frac_bits)
         # Clamp the ideal frac_bits to the other fractional bits, to avoid downscaling afterwards
         frac_bits = tf.minimum(frac_bits, other_frac_bits)
-        # Quantize the inputs with the resulting fractional bits
-        if isinstance(other, FixedPoint):
-            outputs = FixedPoint.quantize(inputs, self.value_bits, frac_bits)
-        else:
-            # If the other is a QFloat, adopt its scales
-            outputs = QFloat.quantize(inputs, self.value_bits, other.scales, frac_bits)
+        # Quantize the inputs with the resulting fractional bits and other scales
+        outputs = QFloat.quantize(inputs, self.value_bits, other.scales, frac_bits)
         # Now, upscale to the other fractional bits
         outputs, shift = outputs.upscale(other_frac_bits, other_value_bits)
         # record quantized weights (including shift and promote)
