@@ -99,8 +99,6 @@ class WeightQuantizer(Quantizer):
         if self.rescale:
             # Evaluate the scales to align on the optimal quantization ranges
             scales = QFloat.optimal_scales(ranges, self.value_bits)
-            # Remove the input-dependent scales from the gradient tape to avoid a loopback
-            scales = tf.stop_gradient(scales)
             # Since we use the optimal quantization ranges [-int_max -1, int_max], the inner
             # FixedPoint can be quantized with exactly zero fractional bits
             assert_equal(0., QFloat.max_frac_bits(self.value_bits, ranges, scales))
@@ -109,8 +107,6 @@ class WeightQuantizer(Quantizer):
         else:
             # Evaluate the maximum fractional bits we can use for the specified range
             frac_bits = FixedPoint.max_frac_bits(self.value_bits, ranges)
-            # Remove the input-dependent fractional bits from the gradient tape to avoid a loopback
-            frac_bits = tf.stop_gradient(frac_bits)
             qweights = FixedPoint.quantize(inputs, self.value_bits, frac_bits)
         # Record the quantized weights (it does nothing if recording is disabled)
         self.qweights(qweights)
