@@ -15,13 +15,14 @@
 # limitations under the License.
 # ******************************************************************************
 
-import re
 import warnings
+import tensorflow as tf
 
 from keras import Model
 
-from .utils import deep_clone_model, insert_layer
-from .transforms import invert_batchnorm_pooling, fold_batchnorms
+from .utils import deep_clone_model, insert_layer, find_layer_config
+from .transforms import (align_rescaling, invert_batchnorm_pooling, fold_batchnorms,
+                         align_relu_max_value)
 from ..layers import (QuantizedDense, QuantizedLayerNormalization, QuantizedAddPositionEmbs,
                       QuantizedClassToken, QuantizedReLU, QuantizedAdd, QDropout,
                       QuantizedReshape, QuantizedAttention, QuantizedConv2D, QuantizedMaxPool2D,
@@ -84,8 +85,7 @@ def _handle_not_quantizable_layers(model):
                     not isinstance(inbound.inbound_layers, Dequantizer)):
                 # Inbound will be quantized, add a Dequantizer after it and return the model
                 inbound_name = inbound.inbound_layers.name
-                model = insert_layer(model, inbound_name,
-                                     Dequantizer(name=f"{inbound_name}_float_output"))
+                model = insert_layer(model, inbound_name, Dequantizer())
                 warnings.warn(f"'{layer.name}' of type {layer.__class__} is not supported to "
                               "quantize, a Dequantizer is added before it and quantization will "
                               "stop at this layer.")
@@ -93,6 +93,21 @@ def _handle_not_quantizable_layers(model):
     return model
 
 
+def _initialize_quantizedrelu_outquantizer(qmodel):
+    """ Initialize QuantizeReLU out_quantizer max_value in the given model.
+
+    Since inputs are clipped to the ReLU max_value, the upper bound of the OutputQuantizer max_value
+    is known.
+
+    Args:
+        qmodel (keras.Model): a quantized model
+    """
+    for ly in qmodel.layers:
+        if isinstance(ly, QuantizedReLU):
+            max_value = tf.fill(ly.out_quantizer.max_value.shape, ly.max_value)
+            OutputQuantizer._assign_new_value(ly.out_quantizer.max_value, max_value)
+
+
 def quantize(model, q_config, add_dequantizer=True):
     """Quantizes the model using the provided configuration.
 
@@ -135,16 +150,6 @@ def quantize(model, q_config, add_dequantizer=True):
     Returns:
         keras.Model: the quantized model
     """
-    # Add "/" at the beginning of the names, to avoid ambiguity (e.g. "Dense" and "DropoutDense")
-    q_config = {key if key[0] == "/" else "/" + key: value for key, value in q_config.items()}
-
-    # This iterates through the <key, config> in the JSON, and creates a regexp based on the key.
-    # The regexp is built by:
-    #   - identifying items with the '/' separator,
-    #   - replace trailing <_N> in each item by a matching pattern,
-    #   - concatenate back items matching patterns.
-    q_config_regex = {re.compile("/" + r"(?:|_[\d]+)/".join(key[1:].split('/')) + r"(?:|_[\d]+)$"):
-                      (key[1:], obj) for key, obj in q_config.items()}
 
     # Layers will no longer be quantized as soon as a Dequantizer is found
     quantization_stopped = False
@@ -198,59 +203,8 @@ def quantize(model, q_config, add_dequantizer=True):
         if quantization_stopped:
             return layer.from_config(config)
 
-        # Replace float layer by quantized layer in three different ways:
-        # 1. Check if layer is in quant_config (to quantize specific layers)
-        layer_name = layer.name if layer.name[0] == "/" else "/" + layer.name
-        match_name = [key for key in q_config.keys() if layer_name.endswith(key)]
-        if len(match_name) > 0:
-            # In case of multiple matches, take the most hierarchical specific one
-            match_name = sorted(match_name, key=lambda x: len(
-                x.split('/')), reverse=True)[0]
-            match_conf = q_config[match_name]
-            return get_quantize_layer(layer, match_conf)
-
-        # 2. Suppose that the layer name has the following format:
-        #    .../ParentLayer<_i>/SubParentLayer<_j>/.../Layer<_k>, where ijk are integers and
-        #    <_i>, <_j>, <_k> are optional.
-        #    Match the pattern and replace the layer with a quantized version.
-        match_tuples = []
-        for rkey, qobj in q_config_regex.items():
-            if rkey.search(layer_name):
-                match_tuples.append(qobj)
-        if len(match_tuples) > 0:
-            # In case of multiple matches, takes the configurations with the highest hierarchy
-            max_hierarchy = max([len(qobj[0].split('/'))
-                                for qobj in match_tuples])
-            match_tuples = {tuple(qobj[0].split('/')[::-1]): qobj[1]
-                            for qobj in match_tuples if len(qobj[0].split('/')) == max_hierarchy}
-            # And then, chooses the configuration with the most detailed sub-indices,
-            # from the end to the beginning.
-            #
-            # For example, if the layer name is: A_1/B_2/C_3/D_4, and the config keys are:
-            # keys = [B_2/C/D, B/C_3/D], the code will search the first exact match from bottom
-            # to top, following:
-            # First step (layer_name[0], keys[0]): if D_4 == D ? No, go to next step
-            # Second step (layer_name[0], keys[1]): if D_4 == D ? No, go to next step
-            # Third step (layer_name[1], keys[0]): if C_3 == C ? No, go to next step
-            # Fourth step (layer_name[1], keys[1]): if C_3 == C_3 ? Yes, match with config[key[1]]
-            #
-            # Note: If only exist this keys in config file, keys[0] will be used in the following
-            # layers: A_i/B_2/C_j/D_k, for all ijk in [0, ..., n] and j != 3.
-            for (idx, pattern) in enumerate(layer_name[1:].split('/')[::-1]):
-                match_tuples_copy = {}
-                for (key, match_conf) in match_tuples.items():
-                    if key[idx] == pattern:
-                        match_tuples_copy[key] = match_conf
-                # If only one match is found, it is used as the configuration
-                if len(match_tuples_copy) == 1:
-                    match_conf = list(match_tuples_copy.values())[0]
-                    break
-                # If length of keys is smaller than hierarchical name, use the last match_conf
-                if idx + 1 >= max_hierarchy:
-                    break
-                # Update match_tuples only when patterns match, in case of a tie
-                if len(match_tuples_copy) > 1:
-                    match_tuples = match_tuples_copy
+        match_conf = find_layer_config(layer.name, q_config)
+        if match_conf:
             return get_quantize_layer(layer, match_conf)
 
         # 3. Special cases for Embedding layers
@@ -266,10 +220,16 @@ def quantize(model, q_config, add_dequantizer=True):
             raise e
         return qlayer
 
+    # Align Rescaling (if needed)
+    model = align_rescaling(model)
+
     # Invert BN <-> Pooling layers and fold BN into their preceding layers
     model = invert_batchnorm_pooling(model)
     model = fold_batchnorms(model)
 
+    # Align ReLU max_value to the maximum integer value the OutputQuantizer can use
+    model = align_relu_max_value(model, q_config)
+
     # Check if the model has not quantizable layers and add a Dequantizer before
     model = _handle_not_quantizable_layers(model)
 
@@ -280,8 +240,15 @@ def quantize(model, q_config, add_dequantizer=True):
     # Append Dequantizer at the end of the model to convert the output to float value
     if (add_dequantizer and not isinstance(new_model.layers[-1], Dequantizer)
             and not quantization_stopped):
-        out = Dequantizer(name="float_output")(out)
-    return Model(new_model.input, out)
+        out = Dequantizer()(out)
+
+    # Build the model
+    qmodel = Model(new_model.input, out)
+
+    # Initialize QuantizeReLU out_quantizer max_value
+    _initialize_quantizedrelu_outquantizer(qmodel)
+
+    return qmodel
 
 
 def dump_config(model):
