@@ -21,36 +21,47 @@ import numpy as np
 
 from keras.layers import Rescaling, Conv2D, Dense
 
-from quantizeml.layers.convolution import PaddedConv2D
-
+from ...layers.convolution import PaddedConv2D
 from ...models.utils import deep_clone_model
 
-__all__ = ["fold_rescaling"]
+__all__ = ["align_rescaling"]
 
 
-def fold_rescaling(model):
-    """Folds the Rescaling layer of the model into the next layer weights.
+def align_rescaling(model):
+    """Aligns the Rescaling layer of the model to make it quantization ready.
+
+    This aligns the Rescaling scale to a single scalar, adjusting the weights of
+    the next layer.
+
+    This also folds the offset into the bias of next layer.
+
+    The resulting Rescaling is therefore compatible with a quantization to a
+    QuantizedRescaling.
+
+    If the source model does not contain a Rescaling or if its Rescaling is already
+    aligned, then the original model is returned.
 
     Args:
-        model (keras.Model): a Keras model to fold
+        model (keras.Model): the source Keras model
 
     Returns:
-        keras.Model: the model with Rescaling layer folded
+        keras.Model: the original model or a new model with Rescaling layer aligned
     """
     supported_dst_layers = [Conv2D, Dense]
 
-    # Create a clone model
-    clone_model = deep_clone_model(model)
-
     # Find the model has a Rescaling layer and check limitations on folding
     rescaling_layer = None
     dst_layer = None
 
-    for ly in clone_model.layers:
+    for ly in model.layers:
         if isinstance(ly, Rescaling):
-            # Folding is limited to single outbound node Rescaling layers
+            scale_per_axis = isinstance(ly.scale, list) and len(ly.scale) > 1
+            if not scale_per_axis and ly.offset == 0:
+                # Rescaling is already aligned: nothing to do
+                break
+            # Alignment is limited to single outbound node Rescaling layers
             if len(ly.outbound_nodes) != 1:
-                raise ValueError("Found a Rescaling layer in the model but with multiple "
+                raise ValueError("Found a non-aligned Rescaling layer in the model with multiple "
                                  "outbounds which is not supported.")
             # Retrieve the destination layer and check its type
             dst_layer = ly.outbound_nodes[0].layer
@@ -64,57 +75,58 @@ def fold_rescaling(model):
             same_conv2d = (isinstance(dst_layer, Conv2D)
                            and dst_layer.get_config()['padding'].lower() == 'same')
             offset_per_axis = isinstance(ly.offset, list) and len(ly.offset) > 1
-            scale_per_axis = isinstance(ly.scale, list) and len(ly.scale) > 1
             if same_conv2d and (offset_per_axis or ly.offset != 0 and scale_per_axis):
-                raise NotImplementedError("Folding with offset per-axis or scale per-axis with an "
-                                          "offset into a Conv2D with 'same' padding is not "
-                                          "supported.")
+                raise NotImplementedError("Folding an offset per-axis or with a scale per-axis "
+                                          "into a Conv2D with 'same' padding is not supported.")
             # Rescaling was found, exit the loop
             rescaling_layer = ly
             break
 
-    # If no Rescaling was found, nothing to do, simply return the original model
+    # If no Rescaling that required an aligment was found
     if rescaling_layer is None:
-        return clone_model
+        # Simply return the original model
+        return model
 
-    # Remove the Rescaling layer so that it will be ignored by clone_model
-    clone_model._self_tracked_trackables.remove(rescaling_layer)
-
-    # Bypass the Rescaling layer by connecting its inbound node to the dst_layer inbound node
-    dst_layer._inbound_nodes[0] = rescaling_layer.inbound_nodes[0]
-
-    # Clone model, rescaling layer will disappear in this step
+    # Clone model
     def replace_layer(layer):
         config = layer.get_config()
-        if layer is dst_layer:
+        if layer.name == dst_layer.name:
             # Force bias if there is a rescaling offset
             if rescaling_layer.offset != 0:
                 config['use_bias'] = True
                 # Replace Conv2D with 'same' padding by PaddedConv2D with correct padding value
                 if isinstance(dst_layer, Conv2D) and config['padding'].lower() == 'same':
-                    # Both offset and scale have a single value at this point
+                    # Offset has a single value at this point
                     offset = rescaling_layer.offset[0] if isinstance(
                         rescaling_layer.offset, list) else rescaling_layer.offset
-                    scale = rescaling_layer.scale[0] if isinstance(
-                        rescaling_layer.scale, list) else rescaling_layer.scale
-                    config['padding_value'] = -offset / scale
+                    config['padding_value'] = float(-offset)
                     return PaddedConv2D.from_config(config)
         return layer.__class__.from_config(config)
 
-    folded_model = deep_clone_model(clone_model, clone_function=replace_layer)
-    folded_layer = folded_model.get_layer(dst_layer.name)
+    aligned_model = deep_clone_model(model, clone_function=replace_layer)
+    rescaling_layer = aligned_model.get_layer(rescaling_layer.name)
+    next_layer = aligned_model.get_layer(dst_layer.name)
 
     # Set weights in the layer of the new model, using the Rescaling parameters:
-    base_weights = dst_layer.get_weights()
+    base_weights = next_layer.get_weights()
     new_w = base_weights[0].copy()
     filters = new_w.shape[-1]
-    # new_weights = base_weights * scale,
-    for i in range(filters):
-        # Rescale weights filter by filter to enable broadcast if scales are per channel
-        new_w[..., i] *= rescaling_layer.scale
+
+    scale = rescaling_layer.scale
+    if isinstance(scale, list) and len(scale) > 1:
+        # If scale is not a scalar, align it
+        target_scale = np.mean(scale)
+        rescaling_layer.scale = target_scale
+        # To compensate, adjust the weights of the next layer
+        for i in range(filters):
+            # Rescale weights filter by filter to enable broadcast
+            new_w[..., i] *= scale / target_scale
+        if isinstance(next_layer, PaddedConv2D):
+            # Also rescale the padding value
+            next_layer._padding_value *= scale / target_scale
     new_weights = [new_w]
 
-    if folded_layer.use_bias:
+    if next_layer.use_bias:
         # Build zero initialized biases if the original layer didn't have any
         new_biases = base_weights[1].copy() if dst_layer.use_bias else np.zeros(filters)
         # new_biases = base_biases + sum(base_weights_per_filter * offset)
@@ -123,6 +135,7 @@ def fold_rescaling(model):
             w_i = base_weights[0][..., i]
             new_biases[i] += np.sum(w_i * rescaling_layer.offset)
         new_weights += [new_biases]
+        rescaling_layer.offset = 0
 
-    folded_layer.set_weights(new_weights)
-    return folded_model
+    next_layer.set_weights(new_weights)
+    return aligned_model
