@@ -18,6 +18,7 @@
 Common utility methods used in quantization models.
 """
 
+import re
 import warnings
 from copy import deepcopy
 from pathlib import Path
@@ -25,12 +26,12 @@ from pathlib import Path
 import numpy as np
 import tensorflow_addons as tfa
 from keras.models import clone_model, load_model as kload_model, Model, Sequential
-from keras.layers import serialize, Layer
+from keras.layers import serialize, Layer, InputLayer, Rescaling
 
-from quantizeml.layers.quantizers import OutputQuantizer, Dequantizer
+from ..layers.quantizers import OutputQuantizer, Dequantizer
 
 __all__ = ['load_model', 'deep_clone_model', 'insert_layer', 'load_weights', 'save_weights',
-           'get_layers_by_type']
+           'get_layers_by_type', 'find_layer_config', 'insert_rescaling']
 
 
 def load_model(model_path, custom_layers={}, compile_model=True):
@@ -67,20 +68,14 @@ def deep_clone_model(model, *args, **kwargs):
     return new_model
 
 
-def insert_layer(model, target_layer_name, new_layer):
+def _insert_layer(model, target_layer_name, new_layer):
     """ Inserts the given layer in the model after the layer with the name target_layer_name.
 
-    Note that new_layer type is restricted to (OutputQuantizer, Dequantizer).
-
     Args:
         model (keras.Model): the model to update
         target_layer_name (str): name of the layer after which to insert a layer
         new_layer (keras.Layer): layer to insert
 
-    Raises:
-        ValueError: when target_layer_name is not found in model or new_layer is not in
-            (OutputQuantizer, Dequantizer)
-
     Returns:
         keras.Model: the new model
     """
@@ -88,11 +83,6 @@ def insert_layer(model, target_layer_name, new_layer):
     if not any(ly.name == target_layer_name for ly in model.layers):
         raise ValueError(f'{target_layer_name} not found in model.')
 
-    # Check added layer type
-    if not isinstance(new_layer, (OutputQuantizer, Dequantizer)):
-        raise ValueError(f'Inserted layer must be of type OutputQuantizer or Dequantizer, \
-                        received {type(new_layer)}.')
-
     # get_config documentation mentions that a copy should be made when planning to modify the
     # config
     config = deepcopy(model.get_config())
@@ -169,6 +159,52 @@ def insert_layer(model, target_layer_name, new_layer):
     return new_model
 
 
+def insert_layer(model, target_layer_name, new_layer):
+    """ Inserts the given layer in the model after the layer with the name target_layer_name.
+
+    Note that new_layer type is restricted to (OutputQuantizer, Dequantizer).
+
+    Args:
+        model (keras.Model): the model to update
+        target_layer_name (str): name of the layer after which to insert a layer
+        new_layer (keras.Layer): layer to insert
+
+    Raises:
+        ValueError: when target_layer_name is not found in model or new_layer is not in
+            (OutputQuantizer, Dequantizer)
+
+    Returns:
+        keras.Model: the new model
+    """
+    # Check added layer type
+    if not isinstance(new_layer, (OutputQuantizer, Dequantizer)):
+        raise ValueError(f'Inserted layer must be of type OutputQuantizer or Dequantizer, \
+                        `received {type(new_layer)}.')
+
+    return _insert_layer(model, target_layer_name, new_layer)
+
+
+def insert_rescaling(model, scale, offset):
+    """ Inserts a Rescaling as first layer of the Model (after the Input)
+
+    Args:
+        model (keras.Model): the model to update
+        scale (float): the Rescaling scale
+        offset (float): the Rescaling offset
+
+    Raises:
+        ValueError: when the Model does not have an Input layer.
+
+    Returns:
+        keras.Model: the new model
+    """
+    first_layer = model.layers[0]
+    if not isinstance(first_layer, InputLayer):
+        raise ValueError("Inserting a Rescaling layer in a Model without an Input layer is not"
+                         " supported.")
+    return _insert_layer(model, first_layer.name, Rescaling(scale, offset))
+
+
 def apply_weights_to_model(model, weights, verbose=True):
     """Loads weights from a dictionary and apply it to a model.
 
@@ -329,3 +365,29 @@ def get_layers_by_type(model, layer_type):
     for layer in model.layers:
         _get_layers(layer, layers)
     return layers
+
+
+def find_layer_config(layer_name, qconfig):
+    """ Extract the layer quantization parameters from the overall quantization config file.
+
+    Args:
+        layer_name (str): the target layer name
+        qconfig (dict): quantization configuration
+
+    Returns:
+        dict: layer quantization configuration
+    """
+    # Extract configuration matching patterns
+    patterns = list(qconfig.keys())
+
+    # First check if we have an exact match
+    if layer_name in patterns:
+        return qconfig[layer_name]
+
+    # Sort patterns by decreasing length and look for a match
+    patterns = sorted(patterns, key=len, reverse=True)
+    for pattern in patterns:
+        if re.search(pattern, layer_name):
+            return qconfig[pattern]
+
+    return None
