@@ -24,13 +24,13 @@ from ..debugging import assert_equal
 class QFloat(QTensor):
     """A Tensor of FixedPoint values and scales representing float numbers
 
-    The QFloat is a dual representation of a float Tensor of float combining
-    FixedPoint values and float scales.
+    The QFloat is a dual representation of a float Tensor combining FixedPoint
+    values and float scales.
 
     The QFloat is typically used to represent float tensors whose quantization
-    range is not aligned on a power-of-two: the original tensor is first divided
-    by the scales to be aligned on power-of-two ranges, then quantized to
-    FixedPoint values.
+    range is not 'optimal' for FixedPoint quantization: the original tensor is
+    first divided by the scales to be aligned on optimal ranges, then quantized
+    to FixedPoint values.
 
     When converting back to float, values are dequantized and multiplied by the
     scales to obtain the approximated float tensor.
@@ -43,7 +43,7 @@ class QFloat(QTensor):
     scales: tf.Tensor = 1.0
 
     def __init__(self, fp, scales):
-        self.scales = scales
+        self.scales = tf.cast(scales, tf.float32)
         self.fp = fp
         self.shape = self.fp.shape
 
